/*****************************************************************************/
/*                                                                           */
/* (c) Copyright IBM Corp. 2002  All rights reserved.                        */
/*                                                                           */
/* This sample program is owned by International Business Machines           */
/* Corporation or one of its subsidiaries ("IBM") and is copyrighted         */
/* and licensed, not sold.                                                   */
/*                                                                           */
/* You may copy, modify, and distribute this sample program in any           */
/* form without payment to IBM,  for any purpose including developing,       */
/* using, marketing or distributing programs that include or are             */
/* derivative works of the sample program.                                   */
/*                                                                           */
/* The sample program is provided to you on an "AS IS" basis, without        */
/* warranty of any kind.  IBM HEREBY  EXPRESSLY DISCLAIMS ALL WARRANTIES,    */
/* EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.       */
/* Some jurisdictions do not allow for the exclusion or limitation of        */
/* implied warranties, so the above limitations or exclusions may not        */
/* apply to you.  IBM shall not be liable for any damages you suffer as      */
/* a result of using, modifying or distributing the sample program or        */
/* its derivatives.                                                          */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* Program name: imsbridge                                                   */
/*                                                                           */
/* Description:  Sample C++ program to illustrate how the imqIMSBridge       */
/*               class can be used to put a message to an Ims Bridge         */
/*               queue. This sample is based on the C program progts2.c      */
/*               contained in SupportPac MA1C.                               */
/*                                                                           */
/*                                                                           */
/* Function:     This program connects to the requested queue manager and    */
/*               queue specified as program arguments. It then creates and   */  
/*               populates the MQMD and MQIIH headers using the imqMessage   */
/*               and imqIMSBridgeHeader classes. Finally, it sets the LL and */
/*               ZZ values along with the message text and puts the message  */
/*               to the requested queue.                                     */
/*                                                                           */
/* Prereqs:      MQSeries V5.2                                               */
/*                                                                           */
/* Setup:        Create a local queue manager and queue with channels        */
/*               communicating with MVS IMS system. Refer to SupportPac      */
/*               MA1C readme file for detail on setup.                       */
/*                                                                           */
/* This program is run as follows:                                           */
/*                                                                           */
/*               imsbridge queue <queuemanagername>                          */
/*                                                                           */
/*                                                                           */
/* Build:                                                                    */
/*               Compile with 'cl imsbridge.cpp imqb23vn.lib imqs23vn.lib'   */
/*               This sample was built using MS Visual C++ V6.0 and tested   */
/*               on Win2000 with MQSeries V5.2 CSD04.                        */
/*                                                                           */
/*****************************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <imqi.hpp>

const int bufsize = 512;

void main (int argc, char** argv)
{                                                                        
   ImqQueueManager    mgr;         // The queue manager.
   ImqQueue           queue;       // IMS bridge message queue.


   if ( argc < 2 )
   {
      cout << "Syntax: " << argv[0] 
           << " queuename <qmanagername>. " 
           << endl;
      exit (100);
   }

   if ( argc > 2 )
   {
      mgr.setName (argv[2]);                                              
   }

   /*********************************************************/
   /* Connect to local queue manager and open the requested */
   /* target queue.                                         */
   /*********************************************************/
   if ( !mgr.connect () )
   {
      cerr << "ImqQueueManager::connect ended with reason code "
           <<  (int)mgr.reasonCode ()
           <<  endl;
      exit ((int)mgr.reasonCode ());                                      
   }

   queue.setConnectionReference (mgr);
   queue.setName (argv[1]);
   queue.setOpenOptions (MQOO_OUTPUT
                         + MQOO_FAIL_IF_QUIESCING);
   queue.open( );
   if ( queue.reasonCode () )
   {
      cerr << "ImqQueue::open ended with reason code "
           << (int)queue.reasonCode () 
           << endl;
   }

   if ( queue.completionCode () == MQCC_FAILED )
   {
      cerr << "unable to open queue for output." << endl;
      mgr.disconnect ();
      exit ((int)mgr.reasonCode ());                                      
   }

   unsigned char ID[MQ_CORREL_ID_LENGTH];
   memset (ID, '\0', sizeof(ID));
   memmove(ID, "IBA0031A", 8);


   /*********************************************************/
   /* Instantiate an imqMessage object and use its methods  */
   /* to populate the MQMD fields.                          */
   /*********************************************************/
   ImqMessage msg;
   msg.setEncoding (MQENC_NATIVE);
   msg.setCharacterSet (MQCCSI_Q_MGR);
   msg.setMessageId (ID);
   msg.setCorrelationId (ID);
   msg.setReport (MQRO_NONE);
   msg.setMessageType (MQMT_REQUEST);
   msg.setExpiry (MQEI_UNLIMITED);
   msg.setFeedback (MQFB_NONE);
   msg.setPriority (MQPRI_PRIORITY_AS_Q_DEF);
   msg.setPersistence (MQPER_PERSISTENCE_AS_Q_DEF);

   msg.setUserId ();
   msg.setAccountingToken ();
   msg.setApplicationIdData ();
   msg.setPutApplicationType (MQAT_DEFAULT);
   msg.setPutApplicationName ();

   msg.setReplyToQueueManagerName (mgr.name ());                                  
   msg.setReplyToQueueName (queue.name ());
   

   /*********************************************************/
   /* Do NOT set the md.format field to MQFMT_IMS. The      */
   /* imqItem copyOut method sets this. It also moves the   */
   /* contents of md.format to mqiih.format                 */
   /* NOTE: Setting fomrat to MQFMT_IMS causes error        */
   /* MQRC_IIS_ERROR (2148).                                */ 
   /*********************************************************/
   //msg.setFormat (MQFMT_IMS);     
   msg.setFormat ("MQIMSVS ");       


   /*********************************************************/
   /* Instantiate an imqIMSBridgeHeader object and use its  */
   /* methods to populate the MQIIH fields.                 */
   /*********************************************************/
   ImqIMSBridgeHeader imshdr;
   imshdr.setAuthenticator ("        ");
   imshdr.setEncoding (MQENC_NATIVE);
   imshdr.setCharacterSet (MQCCSI_Q_MGR);
   imshdr.setHeaderFlags (MQIIH_NONE);
   imshdr.setFormat (MQFMT_IMS_VAR_STRING);
   imshdr.setReplyToFormat (MQFMT_IMS_VAR_STRING);
   imshdr.setLogicalTerminalOverride ("MASTER  ");
   imshdr.setMessageFormatServicesMapName ("MODU03  ");
   imshdr.setSecurityScope (MQISS_CHECK);
   imshdr.setCommitMode (MQICM_COMMIT_THEN_SEND);
   imshdr.setTransactionState (MQITS_NOT_IN_CONVERSATION);
   imshdr.setTransactionInstanceId();


    
   /*********************************************************/
   /* Create a local buffer and allow the imqCache class to */
   /* use it to manage the message buffer                   */
   /*********************************************************/
   char *localbuffer = new char[bufsize];
   msg.useEmptyBuffer (localbuffer, bufsize);

   int offset = 0;          
   short LL = 0;
   short ZZ =0;

   char *data = "IAPMDI28 A Test Message for IMS";              

   /*********************************************************/
   /* The LL field contains the size of the message plus    */
   /* 2 bytes for the LL plus 2 bytes for the ZZ            */
   /* ZZ contains zeros.                                    */
   /*********************************************************/
   LL = strlen(data) + 2 * sizeof(short);                                               

   memmove (localbuffer + offset,&LL,sizeof(short));                                           
   offset += sizeof(short);                                                              
   memmove (localbuffer+offset,&ZZ,sizeof(short));                                        
   offset += sizeof(short);                                                             
   strcpy (localbuffer+offset,data);

   msg.setMessageLength (LL);

   /*********************************************************/
   /* Use the inherited imqItem's copyOut method to insert  */
   /* the MQIIH header at the beginning of the message body.*/
   /*********************************************************/
   if ( !imshdr.copyOut (msg) )
   {
      cerr << "copyOut FAILED with reason code = " 
           << (int)imshdr.reasonCode () 
           << endl;   
   }

   /*********************************************************/
   /* Tell the message cache how long it's message is in    */
   /* preparation for the put. The length is now            */
   /* MQIIH + LL + ZZ + TRANS + TEXT.                       */
   /*********************************************************/

   if ( !queue.put (msg) )
   {
      cerr << "put FAILED with reason code = " 
           << (int)queue.reasonCode () 
           << endl;   
   }
   else
   {
      cout << "Message successfully put to queue " 
           << queue.name () << endl;
   }


   /*********************************************************/
   /* Cleanup the queue and queue manager resources         */
   /*********************************************************/
   queue.close ();
   mgr.disconnect ();

}                                                                        


